﻿using System;
using System.Collections.Generic;
using Apache.NMS;
using ProjectBase.Data.Logs;
using SIMDP.Device;
using SIMDP.Util;
using static SIMDP.Project.ActiveMQ;

namespace SIMDP.Project
{
    class OpcUaDemo : PlcOpcUA
    {
        ActiveMQ mymq = new ActiveMQ();

        #region 重写父类函数
        public override void actionStart()
        {
            mymq.Connect(isLocalMachine: true, remoteAddress: "");
            dict.Clear();
            mymq.InitQueueOrTopic(false, "myqueue1");
            mymq.InitQueueOrTopic(true, "mytopic3");
            dict["mytopic3"].consumer.Listener += new MessageListener(Consumer_Listener);
        }

        public override void actionStop()
        {
            mymq?.DisConnect();
        }
        public override void actionTimer()
        {
            try
            {
                foreach (var p in monitorPoints)
                {
                    if (!Read(p, out object value)) continue;
                    if (!value.Equals(p.value))// != p.value)
                    {
                        p.value = value;
                        if (p.proc != null)
                            p.proc.Invoke(this, new object[] { value });
                    }
                }
                redis.Publish(SysEnvironment.OpcDataChannel, "");
            }
            catch (Exception ex)
            {
                LogHelper.log.Error("读取OPCUA(" + m_OpcUaClient.OpcUaName + ")信号(" + ")发生异常:" + ex.Message);
                m_OpcUaClient.Disconnect();
                SetRedisPlcDisConnect();
            }
        }
        #endregion 重写父类函数

        #region UA信号函数
        public void UAtoTopic(object value)
        {
            bool signal = Convert.ToBoolean(value);
            if (!signal)
                return;

            //mymq.SendMessage("message", dict["myqueue1"]);
            mymq.SendMessage("ua_topic", dict["mytopic3"]);
        }

        #endregion UA信号函数

        #region Topic监听事件
        void Consumer_Listener(IMessage message)
        {
            ITextMessage msg = (ITextMessage)message;
            Console.WriteLine($"通过监听事件获取到{msg.Text}"+"\n");
            LogHelper.log.Debug($"通过监听事件获取到{msg.Text}"+"\n");

            string str = "topic_ua";
            Write("v03", str);
            Console.WriteLine($"写入OPCUA变量：{str}");
            LogHelper.log.Debug($"写入OPCUA变量：{str}");
        }

        #endregion Topic监听事件
    }
}
