﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Linq;
using System.Threading.Tasks;
using System.Windows.Forms;
using DevExpress.XtraEditors;
using ProjectBase.Data.Logs;
using SIMDP.Util;
using SIMDP.Model;
using SIMDP.BLL;
using ProjectBase.Data.BaseDAL;

namespace SIMDP
{
    public partial class FormLogLogIn : DevExpress.XtraEditors.XtraForm
    {
        //BlAuthoryGroup blGroup = new BlAuthoryGroup();
        //BlLogLogIn blLogIn = new BlLogLogIn();
        string condition = null;

        public FormLogLogIn()
        {
            InitializeComponent();
        }

        private void FormLogLogIn_Load(object sender, EventArgs e)
        {
            InitTree();
            InitGrid();
        }

        /// <summary>
        /// 初始化树
        /// </summary>
        private void InitTree()
        {
            this.tree_LogIn.BeginUpdate();
            this.tree_LogIn.Nodes.Clear();

            List<MoAuthoryGroup> list = BLLFactory<BlAuthoryGroup>.Instance.GetAll();
            List<MoAuthoryGroup> rootList = new List<MoAuthoryGroup>();

            foreach (MoAuthoryGroup item in list)
            {
                if (item.ParentGroupId == "")
                {
                    rootList.Add(item);
                }
            }

            foreach (MoAuthoryGroup root in rootList)
            {
                if (root != null)
                {
                    TreeNode rootNode = new TreeNode();   //根节点
                    rootNode.Text = root.GroupName;
                    rootNode.Name = root.GroupId;
                    rootNode.Tag = string.Format(" logIn_groupId = '{0}'", root.GroupId);
                    rootNode.ImageIndex = 0;
                    rootNode.SelectedImageIndex = 0;
                    this.tree_LogIn.Nodes.Add(rootNode);

                    List<MoAuthoryGroup> subList = BLLFactory<BlAuthoryGroup>.Instance.Find(string.Format(" parent_group_id = '{0}'", root.GroupId));
                    AddNode(subList, rootNode);
                }
            }

            this.tree_LogIn.ExpandAll();
            this.tree_LogIn.EndUpdate();
            this.tree_LogIn.SelectedNode = null;
            this.tree_LogIn.TabStop = false;   //移除焦点
        }

        /// <summary>
        /// 添加树节点
        /// </summary>
        /// <param name="list"></param>
        /// <param name="parentNode"></param>
        private void AddNode(List<MoAuthoryGroup> list, TreeNode parentNode)
        {
            if (list == null)
            {
                return;
            }
            foreach (MoAuthoryGroup item in list)
            {
                TreeNode node = new TreeNode();   //根节点
                node.Text = item.GroupName;
                node.Name = item.GroupId;
                node.Tag = string.Format(" logIn_groupId = '{0}'", item.GroupId);
                string index = item.GroupId.Substring(0, 1);
                node.ImageIndex = Convert.ToInt32(index) - 1;
                node.SelectedImageIndex = Convert.ToInt32(index) - 1;
                parentNode.Nodes.Add(node);

                List<MoAuthoryGroup> subList = BLLFactory<BlAuthoryGroup>.Instance.Find(string.Format(" parent_group_id = '{0}'", item.GroupId));
                AddNode(subList, node);
            }

        }

        /// <summary>
        /// 选中节点后显示
        /// </summary>
        /// <param name="sender"></param>
        /// <param name="e"></param>
        private void tree_LogIn_AfterSelect(object sender, TreeViewEventArgs e)
        {
            if (e.Node != null && e.Node.Tag != null)
            {
                condition = e.Node.Tag.ToString();
                BindData(condition);
            }
            else
            {
                condition = " 1=1";
                BindData(condition);
            }
        }

        /// <summary>
        /// 初始化表格控件
        /// </summary>
        private void InitGrid()
        {
            //entity
            this.winGridViewPager1.OnPageChanged += new EventHandler(winGridViewPager1_OnPageChanged);
            this.winGridViewPager1.OnStartExport += new EventHandler(winGridViewPager1_OnStartExport);
            this.winGridViewPager1.OnRefresh += new EventHandler(winGridViewPager1_OnRefresh);
            this.winGridViewPager1.gridView1.CustomColumnDisplayText += new DevExpress.XtraGrid.Views.Base.CustomColumnDisplayTextEventHandler(gridView1_CustomColumnDisplayText);
            this.winGridViewPager1.ShowLineNumber = true;
            this.winGridViewPager1.BestFitColumnWith = true;
            this.winGridViewPager1.DisplayColumns = "Account,Name,GroupId,Description,Ip,Mac,Time";
            #region 添加别名解析

            this.winGridViewPager1.AddColumnAlias("ID", "");
            this.winGridViewPager1.AddColumnAlias("Account", "登录用户账号");
            this.winGridViewPager1.AddColumnAlias("Name", "登录用户名称");
            this.winGridViewPager1.AddColumnAlias("GroupId", "所属公司");
            this.winGridViewPager1.AddColumnAlias("Description", "日志描述");
            this.winGridViewPager1.AddColumnAlias("Ip", "IP地址");
            this.winGridViewPager1.AddColumnAlias("Mac", "Mac地址");
            this.winGridViewPager1.AddColumnAlias("Time", "日志更新时间");

            #endregion
            this.winGridViewPager1.PrintTitle = "用户登录日志表";
            this.winGridViewPager1.gridView1.Appearance.HeaderPanel.TextOptions.HAlignment = DevExpress.Utils.HorzAlignment.Center;

            condition = " 1=1";
            BindData(condition);
        }

        /// <summary>
        /// 绑定数据源
        /// </summary>
        /// <param name="condition"></param>
        private void BindData(string condition)
        {
            this.winGridViewPager1.DataSource = BLLFactory<BlLogLogIn>.Instance.FindWithPager(condition, this.winGridViewPager1.PagerInfo);
            for (int i = 0; i < this.winGridViewPager1.gridView1.Columns.Count; i++)   //设置每列内容居中显示
            {
                this.winGridViewPager1.gridView1.Columns[i].AppearanceCell.TextOptions.HAlignment = DevExpress.Utils.HorzAlignment.Center;
            }
        }

        /// <summary>
        /// 分页控件翻页的操作
        /// </summary> 
        private void winGridViewPager1_OnPageChanged(object sender, EventArgs e)
        {
            BindData(condition);
        }

        /// <summary>
        /// 分页控件全部导出操作前的操作
        /// </summary> 
        private void winGridViewPager1_OnStartExport(object sender, EventArgs e)
        {
            this.winGridViewPager1.AllToExport = BLLFactory<BlLogLogIn>.Instance.Find(condition);
        }

        /// <summary>
        /// 分页控件刷新操作
        /// </summary>
        private void winGridViewPager1_OnRefresh(object sender, EventArgs e)
        {
            BindData(condition);
        }

        void gridView1_CustomColumnDisplayText(object sender, DevExpress.XtraGrid.Views.Base.CustomColumnDisplayTextEventArgs e)
        {
            if (e.Column.ColumnType == typeof(DateTime))
            {
                string columnName = e.Column.FieldName;
                if (e.Value != null)
                {
                    if (Convert.ToDateTime(e.Value) <= Convert.ToDateTime("1900-1-1"))
                    {
                        e.DisplayText = "";
                    }
                    else
                    {
                        e.DisplayText = Convert.ToDateTime(e.Value).ToString("yyyy-MM-dd HH:mm");//yyyy-MM-dd
                    }
                }
            }

            Dictionary<string, string> dic = BLLFactory<BlLogLogIn>.Instance.GetGroupName();
            if (e.Column.FieldName == "GroupId")
            {
                e.DisplayText = dic.FirstOrDefault(p => p.Key == e.Value.ToString().Trim()).Value;
            }
        }

        private void menu_Expand_Click(object sender, EventArgs e)
        {
            this.tree_LogIn.ExpandAll();
        }

        private void menu_Collapse_Click(object sender, EventArgs e)
        {
            this.tree_LogIn.CollapseAll();
        }

        private void menu_Refresh_Click(object sender, EventArgs e)
        {
            InitTree();
        }

        private void btn_Query_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(txt_Account.Text) && string.IsNullOrEmpty(txt_UserName.Text) && string.IsNullOrEmpty(txt_IP.Text) && string.IsNullOrEmpty(txt_Mac.Text)
                 && string.IsNullOrEmpty(txt_Des.Text) && string.IsNullOrEmpty(date_Start.Text) && string.IsNullOrEmpty(date_End.Text))
            {
                BindData(" 1=1");
                condition = " 1=1";
                return;
            }

            string sql = " 1=1";
            if (!string.IsNullOrEmpty(txt_Account.Text))
            {
                sql += string.Format(" AND logIn_account = '{0}'",txt_Account.Text);
            }

            if (!string.IsNullOrEmpty(txt_UserName.Text))
            {
                sql += string.Format(" AND logIn_name = '{0}'",txt_UserName.Text);
            }

            if (!string.IsNullOrEmpty(txt_IP.Text))
            {
                sql += string.Format(" AND logIn_ip = '{0}'",txt_IP.Text);
            }

            if (!string.IsNullOrEmpty(txt_Mac.Text))
            {
                sql += string.Format(" AND logIn_mac = '{0}'",txt_Mac.Text);
            }

            if (!string.IsNullOrEmpty(date_Start.Text))
            {
                sql += string.Format(" AND logIn_time >= '{0}'", date_Start.DateTime.ToString("yyyy-MM-dd"));
            }

            if (!string.IsNullOrEmpty(date_End.Text))
            {
                sql += string.Format(" AND logIn_time < '{0}'", date_End.DateTime.AddDays(1).ToString("yyyy-MM-dd"));
            }

            BindData(sql);
            condition = sql;
        }

    }
}